<?php
namespace Ems\portalGestor\Controller;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;

use Ems\portalGestor\Service\ControleEntregaEPIService;


class ControleEntregaEPIController extends Crud {
    /** @var ControleEntregaEPIService */
    protected $controleEntregaEPIService;

    public function __construct(
        EntityManager $entityManager,
        ControleEntregaEPIService $controleEntregaEPIService
    ) {
        $this->entityManager             = $entityManager;
        $this->controleEntregaEPIService = $controleEntregaEPIService;
    }

    public function getControleEntregaEPI(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();
            $controleEntregaEPI = $this->controleEntregaEPIService->getControleEntregaEPI(
                $row
            );

            $response->addDataSet(new DataSet('data', $controleEntregaEPI));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getEpi(DTO\Request\Filter $request, Response $response) {
        try {
            $params = $request->getParameters();
            $nrOrg  = $params['NRORG'];

            $dataSet = $this->controleEntregaEPIService->getEpi($nrOrg);
            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function setHistoricoEpi(
        Request\Row $request,
        Response $response
    ){
        try {
            $row = $request->getRow();
            $success = $this->controleEntregaEPIService->setHistoricoEpi($row);

            $dataset = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('data', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function deleteHistoricoEpi(
        Request\Row $request,
        Response $response
    ){
        try {
            $row = $request->getRow();
            $success = $this->controleEntregaEPIService->deleteHistoricoEpi($row);

            $dataset = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('data', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

}